
/******************************************************************************\
*       This is a part of the Microsoft Source Code Samples. 
*       Copyright (C) 1994-1996 Microsoft Corporation.
*       All rights reserved. 
*       This source code is only intended as a supplement to 
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the 
*       Microsoft samples programs.
\******************************************************************************/

/*++

Module Name:

    kill.c

Abstract:

    This module implements a task killer application.

--*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../common.h"


#define MAX_TASKS           256

BOOL        ForceKill;
DWORD       pid;
WCHAR        pname[MAX_PATH];
TASK_LIST   tlist[MAX_TASKS];


VOID GetCommandLineArgs(VOID);
VOID Usage(VOID);



int _cdecl
wmain(
    int argc,
    WCHAR *argv[]
    )
{
    DWORD             i;
    DWORD             numTasks;
    TASK_LIST_ENUM    te;
    int               rval = 0;
    WCHAR             tname[PROCESS_SIZE];
    LPWSTR             p;
    DWORD             ThisPid;
    OSVERSIONINFO     verInfo = {0};


    GetCommandLineArgs();

    if (pid == 0 && pname[0] == 0) {
        wprintf( L"missing pid or task name\n"
                 L"type kill /? for help\n");
		return 1;
    }

    if (pid) {
        tlist[0].dwProcessId = pid;
        if (KillProcess( tlist, TRUE )) {
            wprintf( L"process #%x killed\n", pid );
            return 0;
        } else {
            wprintf( L"process #%x could not be killed\n", pid );
            return 1;
        }
    }

    //
    // get the task list for the system
    //
    numTasks = GetTaskList( tlist, MAX_TASKS );

    //
    // enumerate all windows and try to get the window
    // titles for each task
    //
    te.tlist = tlist;
    te.numtasks = numTasks;
    GetWindowTitles( &te );

    ThisPid = GetCurrentProcessId();

    for (i=0; i<numTasks; i++) {
        //
        // this prevents the user from killing KILL.EXE and
        // it's parent cmd window too
        //
        if (ThisPid == tlist[i].dwProcessId) {
            continue;
        }
        if (MatchPattern( tlist[i].WindowTitle, L"*KILL*" )) {
            continue;
        }

        tname[0] = 0;
        wcscpy( tname, tlist[i].ProcessName );
        p = wcschr( tname, L'.' );
        if (p) {
            p[0] = L'\0';
        }
        if (MatchPattern( tname, pname )) {
            tlist[i].flags = TRUE;
        } else if (MatchPattern( tlist[i].ProcessName, pname )) {
            tlist[i].flags = TRUE;
        } else if (MatchPattern( tlist[i].WindowTitle, pname )) {
            tlist[i].flags = TRUE;
        }
    }

    for (i=0; i<numTasks; i++) {
        if (tlist[i].flags) {
            if (KillProcess( &tlist[i], ForceKill )) {
                wprintf( L"process #%x [%s] killed\n", tlist[i].dwProcessId, tlist[i].ProcessName );
            } else {
                wprintf( L"process #%x [%s] could not be killed\n", tlist[i].dwProcessId, tlist[i].ProcessName );
                rval = 1;
            }
        }
    }

    return rval;
}

VOID
GetCommandLineArgs(
    VOID
    )
{
    WCHAR       *lpstrCmd;
    WCHAR       ch;
    WCHAR       *p = pname;


    pid = 0;
    *p = L'\0';

    lpstrCmd = GetCommandLine();



    // Follow this pattern strictly:
    //  KILL [options] <<pid> | <pattern>>\n\n" 
    //  where options is /f,
    //  and PID is a positive or negative decimal number, and
    //  pattern is a string of characters

    // Look for an option, which will be prefaced with '/'
	ch = *lpstrCmd;
    if (ch == L'/')
    {
       ch = *++lpstrCmd;
  
       switch (ch) 
       {
       case L'F':     // found "kill /f" so far
       case L'f':
          ForceKill = TRUE;
          ch = *++lpstrCmd;
          break;

       case L'?':    // found "kill /?"
          Usage();
          return;   // don't do anything after printing usage
          break;

       default:
          wprintf (L"%c is an invalid switch\n", ch);  // invalid switch
          return;   
       }
    
       // eat whitepace after option switch
       while (iswspace(ch)) 
       {
          ch = *++lpstrCmd;
       }
	   
    }

    // Look for a postive PID
	if(iswdigit(ch))
	{
		pid = wcstoul(lpstrCmd,NULL,16);
		return;  // found "kill pid" or "kill /f pid" so we're done
    }
 
    // Look for a pattern
    while (ch) 
    {
       *p++ = ch;
       ch = *++lpstrCmd;
    }
    *p = L'\0';
    _wcsupr(pname);

    return;  // found "kill pattern" or "kill /f pattern" so we're done
}
      


VOID
Usage(
    VOID
    )

/*++

Routine Description:

    Prints usage text for this tool.

Arguments:

    None.

Return Value:

    None.

--*/

{
    fwprintf( stderr, L"Microsoft (R) Windows (TM) KILL\n" );
    fwprintf( stderr, L"Copyright (C) 1994-1996 Microsoft Corp. All rights reserved\n" );
    fwprintf( stderr, L"usage: KILL [options] <<pid> | <pattern>>\n" );
    fwprintf( stderr, L"[options]:\n" );
    fwprintf( stderr, L"	/?     This help\n" );
    fwprintf( stderr, L"	/f     Force process kill\n" );
    fwprintf( stderr, L"<pid>\n" );
    fwprintf( stderr, L"	This is the process id for the task\n" );
    fwprintf( stderr, L"	to be killed.  Use TLIST to get a valid\n" );
    fwprintf( stderr, L"	pid. Use leading zero for pid.\n" );
    fwprintf( stderr, L"<pattern>\n" );
    fwprintf( stderr, L"	The pattern can be a complete task\n" );
    fwprintf( stderr, L"	name or a regular expression pattern\n" );
    fwprintf( stderr, L"	to use as a match.  Kill matches the\n" );
    fwprintf( stderr, L"	supplied pattern against the task names\n" );
    fwprintf( stderr, L"	and the window titles.\n" );
    exit(0);
}
